<?php
/**
 * Copyright (©) 2020 Pinpoint Designs LTD. All rights reserved.
 *
 * Author: Chandravadana Kandregula
 * Code Release Version: 1.0.0
 *
 * Class Data
 * @module Pinpoint_DispatchTimer
 */

namespace Pinpoint\DispatchTimer\Helper;

use Magento\Catalog\Helper\Data as catalogHelper;
use Magento\Catalog\Model\Product;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Framework\Stdlib\DateTime\TimezoneInterface;
use Magento\Store\Model\ScopeInterface;

class Data extends AbstractHelper
{
    const SHIPPING_MESSAGE = 'shipping_message';

    /** @var Json */
    private $json;

    /** @var TimezoneInterface */
    private $timezone;

    /** @var string */
    private static $configPathPrefix = 'pinpoint/dispatch_timer/';

    /**
     * Data constructor.
     * @param Context $context
     * @param Json $json
     * @param TimezoneInterface $timezone
     */
    public function __construct(
        Context $context,
        Json $json,
        TimezoneInterface $timezone
    ) {
        $this->json = $json;
        $this->timezone = $timezone;

        parent::__construct($context);
    }

    /**
     * Get the Dispatch Timer status.
     *
     * @return mixed
     */
    public function getDispatchTimerStatus()
    {
        return $this->scopeConfig->getValue(
            self::$configPathPrefix . 'status',
            ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * Get the cut-off time as an array.
     *
     * @return array
     */
    public function getCutOffTime()
    {
        $configValue = $this->scopeConfig->getValue(
            self::$configPathPrefix . 'cutoff_time',
            ScopeInterface::SCOPE_STORE
        );

        return explode(',', $configValue);
    }

    /**
     * Get the dispatch days as an array.
     *
     * @return array
     */
    public function getDispatchDays()
    {
        $configValue = $this->scopeConfig->getValue(
            self::$configPathPrefix . 'dispatch_days',
            ScopeInterface::SCOPE_STORE
        );

        return explode(',', $configValue);
    }

    /**
     * Get the number of days to ship.
     *
     * @return mixed
     */
    public function getDaysToShip()
    {
        return $this->scopeConfig->getValue(
            self::$configPathPrefix . 'days_to_ship',
            ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * Get the list of holidays as formatted date strings.
     *
     * @return array
     */
    public function getHolidays()
    {
        $unavailableDates = $this->getUnavailableDates();
        $cutOffTime = $this->getCutOffTime();
        $holidays = [];
        foreach ($unavailableDates as $uDate) {
            $holidays[] = date(
                str_pad($uDate['year'], 4, '20', STR_PAD_LEFT) . '-'
                . str_pad($uDate['month'], 2, '0', STR_PAD_LEFT) . '-'
                . str_pad($uDate['day'], 2, '0', STR_PAD_LEFT) . ' '
                . $cutOffTime[0] . ':' . $cutOffTime[1] . ':' . $cutOffTime[2]
            );
        }

        return $holidays;
    }

    /**
     * Get the cut-off date and time as a formatted string.
     *
     * @return false|string
     */
    public function getCutOffDateTime()
    {
        $cutOffTime = $this->getCutOffTime();
        return date('Y-m-d ' . $cutOffTime[0] . ':' . $cutOffTime[1] . ':' . $cutOffTime[2], $this->timezone->scopeTimeStamp());
    }

    /**
     * Get the cut-off time as a formatted string.
     *
     * @return false|string
     */
    public function getCutOffTimeFormatted()
    {
        $cutOffTime = $this->getCutOffTime();
        return date($cutOffTime[0] . ':' . $cutOffTime[1] . ':' . $cutOffTime[2], $this->timezone->scopeTimeStamp());
    }

    /**
     * Get the list of unavailable dates as an array.
     *
     * @return array
     */
    public function getUnavailableDates()
    {
        $configValue = $this->scopeConfig->getValue(
            self::$configPathPrefix . 'unavailable_dates',
            ScopeInterface::SCOPE_STORE
        );
        $unserialized = $this->json->unserialize($configValue);
        $dates = [];
        foreach ($unserialized as $dateData) {
            $date = [];
            $date['year'] = $dateData['year'];
            $date['month'] = $dateData['month'];
            $date['day'] = $dateData['day'];
            $dates[] = $date;
        }

        return $dates;
    }

    /**
     * Get an array of days of the week.
     *
     * @return array
     */
    public function getDaysOfWeek()
    {
        return [
            0 => __('Sunday'),
            1 => __('Monday'),
            2 => __('Tuesday'),
            3 => __('Wednesday'),
            4 => __('Thursday'),
            5 => __('Friday'),
            6 => __('Saturday'),
        ];
    }

    /**
     * Get the shipping message.
     *
     * @return mixed|string
     */
    public function getShippingMessage()
    {
        $configValue = $this->scopeConfig->getValue(
            self::$configPathPrefix . self::SHIPPING_MESSAGE,
            ScopeInterface::SCOPE_STORE
        );
        if (!$configValue) {
            $configValue = 'For Delivery on {day}, order within';
        }
        return $configValue;
    }

    /**
     * Get the standard message.
     *
     * @return mixed|string
     */
    public function getStandardMessage()
    {
        $configValue = $this->scopeConfig->getValue(
            self::$configPathPrefix . 'shipping_message_no_timer',
            ScopeInterface::SCOPE_STORE
        );
        if (!$configValue) {
            $configValue = 'UK next working day delivery is available on this product';
        }
        return $configValue;
    }

    /**
     * Get an array of SKU values for standard messages.
     *
     * @return array|string
     */
    public function getStandMessageSkus()
    {
        $configValue = explode(',', $this->scopeConfig->getValue(
            self::$configPathPrefix . 'product_skus',
            ScopeInterface::SCOPE_STORE
        ));
        if (!$configValue) {
            $configValue = '';
        }
        return $configValue;
    }

    /**
     * Get an array of SKU values to skip.
     *
     * @return string|array
     */
    public function getSkipSkus()
    {
        $configValue = explode(',', $this->scopeConfig->getValue(
            self::$configPathPrefix . 'skip_skus',
            ScopeInterface::SCOPE_STORE
        ));
        if (!$configValue) {
            $configValue = '';
        }
        return $configValue;
    }

    /**
     * Get the label for the shipping message.
     *
     * @return string
     */
    public function getLabel()
    {
        return str_replace('_', ' ', ucwords(self::SHIPPING_MESSAGE, '_'));
    }
}
